#!/usr/bin/perl -w
package SmartClient::CheckFile;
use strict;
use warnings;
use Carp;

=head1 NAME

=begin comment

Name des Moduls,Grobkonzept

=end comment

B<SmartClient::CheckFile>

=head1 SYNOPSIS

=begin comment

  kurze Beschreibung, was das Programm macht, Feinkonzept

=end comment

Objekt zur Ueberprüfung des Änderungsdatum einer Datei.

 SmartClient::CheckFile->new(FILE => "Dateiname");
 
 SmartClient::CheckFile->check();

=head1 DESCRIPTION

=begin comment

  muss fuer das Grobkonzept ausgefuellt werden

=end comment

=head2 Allg. Beschreibung und Aufgaben

Modul überprüft das Änderungsdatum einer Datei. Es stellt eine Methode zur Verfügung, die erst einen Wert zurückliefert wenn sich eine angegebene Datei verändert hat. Der Rhythmus der Überpüfung kann angegeben werden.

=head2 Funktionsweise / Design / Architektur / Sprache

=over 4

=item Funktionsweise

B<SmartClient::CheckFile> bildet eine Instanz eines Dateiobjektes,
wenn Informationen über die betroffene Datei mittels der stat-Funktion gesammelt werden kann. Als wichtiger Parameter wird das Änderungsdatum der Datei in einer Variablen gespreichert.
Wird anschließend die Methode check aufgerufen, wird dort in definierbaren Zeitabständen das Änderungsdatum der entsprechenden Datei überprüft. Wird eine Änderung detektiert, kehrt die Methode zurück.




=item Architektur

Linux i386


=item Sprache

  Perl

=back

=head2 Sicherheitskonzept

keine Angabe

=head2 Einordnung / Abhängigkeiten / Voraussetzungen

=begin comment

Abhaenigkeiten von anderen Perlmodulen werden durch
require Statements sichergestellt.

=end comment

  L<File::stat>

=head2 Interfaces(Programm/Benutzer)

Dateiname und Überpüfungsrhythmus können durch Parameter gesteuert werden.

=cut

our $VERSION = 0.1;

=head2 CVS Version

$Id: CheckFile.pm 7468 2011-10-07 11:38:20Z FSchumacher $

=cut

=head2 Installation und sonstige Hinweise

=begin comment

Feinkonzept

=end comment

Modul muß im @INC -Pfad stehen.

=head1 AUTHOR

=begin comment

name and e-mail

=end comment

Frank Balzer mailto:frank.balzer@suse.de

SuSE Linux Solutions AG mailto: rheinruhr@suse.de

=head1 SEE ALSO


=cut

=head1 exportierte Module, Methoden, Variablen

=over 4

=item Variable

=item Methoden

S<  >B<check>

=back

=cut

=head1 Modul

=begin comment

Beschreibung des Moduls

=end comment

S<  >B<SmartClient::CheckFile> { I<FILE> =E<gt> I<filename>, I<TIMEOUT> E<gt> I<timeout>};

 # Beispiel

 use SmartClient::CheckFile;

 $obj1 = SmartClient::CheckFile->new(FILE => "/var/log/messages");
 $obj2 = SmartClient::CheckFile->new(FILE => "/var/log/messages", TIMEOUT => 10);

=cut

=head1 Konstanten

$DEFAULT_TIME_OUT = 60 seconds;

=cut

#constants

my $DEFAULT_TIMEOUT        = 60;
my $DEFAULT_GLOBAL_TIMEOUT = 0;

# constructor

use File::stat;

sub new
{
    my $package = shift;
    my $data    = {
        'FILE'          => undef,
        'TIMEOUT'       => undef,
        'GLOBALTIMEOUT' => undef,
        @_
    };
    if ( !$data->{'FILE'} ) {
        return;
    }
    if ( !$data->{'TIMEOUT'} ) {
        $data->{'TIMEOUT'} = $DEFAULT_TIMEOUT;
    }
    if ( !$data->{'GLOBALTIMEOUT'} ) {
        $data->{'GLOBALTIMEOUT'} = $DEFAULT_GLOBAL_TIMEOUT;
    }
    my $filedata = stat( $data->{'FILE'} ) or croak "Can't stat file: $!";
    $data->{'FILETIME'} = $filedata->ctime;

    return bless($data, $package);
}

=head1 Methoden

=head2 check()

S<  >B<check()>

Methode zum überprüfen des Änderungsdatum einer Datei. Die Methode liefert erst einen Wert zurück, wenn sich die zu überwachende Datei ändert.

=cut

sub check
{

    my ($this)  = @_;
    my $gtime   = 0;
    my $timesav = time();
    $this->{'GLOBALTIMEOUT'}
      ? ( $gtime = $this->{'GLOBALTIMEOUT'} )
      : ( $gtime = 0 );
    while (1) {

        #     print $gtime . "\n";
        #     print  $this->{'GLOBALTIMEOUT'} . "\n";
        if ($gtime) {
            if ( $gtime <= ( time() - $timesav ) ) {

                #	      print "Global Timeout . \n";
                return 0;
            }
        }
        my $filedata = stat( $this->{'FILE'} ) or croak "Can\'t stat file: $!";
        if ( $this->{'FILETIME'} != $filedata->ctime ) {
            $this->{'FILETIME'} = $filedata->ctime;
            return 1;
        }

        #    print "sleeping for " . $this->{'TIMEOUT'} . "\n";
        sleep( $this->{'TIMEOUT'} );
    }
    return 0;
}

1;

