#!/usr/bin/perl -w

package SmartClient::CheckUtil;
use strict;
use warnings;

=head1 NAME

SmartClient::CheckUtil

=head2 SVN Version

 $Id: CheckUtil.pm 7471 2011-10-07 12:45:39Z FSchumacher $

=begin comment

Grobkonzept

=end comment

B<SmartClient::CheckUtil>

=head1 SYNOPSIS

=begin comment

  kurze Beschreibung, was das Programm macht, Feinkonzept

=end comment

Hilfsfunktionen zur �erprfung von diversen Daten, wie z.B. Korrektheit
einer IP-Adresse, MAC-Adresse usw.

=head1 DESCRIPTION

=begin comment 

  muss fuer das Grobkonzept ausgefuellt werden

=end comment

=head2 Allg. Beschreibung und Aufgaben

=head2 Funktionsweise / Design / Architektur / Sprache

=over 4

=item Funktionsweise

B<SmartClient::CheckUtil> umfa� Funktionen, die von verschiedenen
Scripten verwendet werden.

=item Architektur

Linux, i386

=begin comment

# Allgemein

=end comment

=item Sprache

  Perl

=back



=head2 Sicherheitskonzept

=head2 Einordnung / Abhaengigkeiten / Voraussetzungen

=begin comment

Abhaenigkeiten von anderen Perlmodulen werden durch
require Statements sichergestellt.

=end comment

=head2 Interfaces(Programm/Benutzer)

=head2 Status (Fehler)

=head2 Version

=begin comment

  Versionsnummer fuer das Modul,  Feinkonzept,

=end comment

=cut

# version  of the package

our $VERSION = 0.1;

=head2 CVS Version

  $Id: CheckUtil.pm 7471 2011-10-07 12:45:39Z FSchumacher $

=head2 Installation und sonstige Hinweise

=begin comment

Feinkonzept

=end comment

=head1 AUTHOR

=begin comment

Name und e-mail

=end comment

Stephan Duehr

=head1 SEE ALSO

=cut

# exportieren der Module
use base qw(Exporter);

use Log::Log4perl;

# Festlegen der zu exportierenden Module, Methoden, Funktionen, Variablen, Konstanten
our @EXPORT = qw(chk_mac_addr chk_split_dhcp_range chk_is_lcase checkIp chk_ip);

=head1 exportierte Module, Methoden, Variablen und Funktionen

=over 4

=item Funktionen

=over 4

=item chk_mac_addr

Prfung auf korrekte MAC-Adresse

=back

=back

=cut

# Festlegen der optional zu exportierenden Module, Methoden, Funktionen, Variablen, Kostanten

#@EXPORT_OK;

=head1 optional exportierte Module

S<  >B<Modul>

S<  >B<Funktion>

S<  >B<Variable>

=cut

=head1 Modul

=begin comment

Beschreibung des Moduls

=end comment

keine Angabe

=cut

#Programmcode

# =head1 Methoden

# =head2 Methode1

# S<  >B<Methode> { I<parameter> =E<gt> I<value>, ... };

# S<  >B<Methode> I<\@param1>, I<\@param2>[, I<$option1> ] [, I<$option2> ] ];

=head1 Funktionen

=head2 Funktion chk_mac_addr

Prfen einer MAC-Adresse auf Korrektheit:

 - 6 Hexadezimalwerte durch : getrennt
 - Gro�/Kleinschreibung der Buchstaben der Hexadezimalwerte ist dabei
   irrelevant

=over 4

=item Parameter

 MAC-Adresse (String)

Beispiel:

 chk_mac_addr("12:34:56:ab:ae:ff")

=item Rckgabe

True (1), falls MAC-Adresse korrekt, sonst False (0).

=back

=cut

sub chk_mac_addr
{
    my ($mac_addr) = @_;

    # MAC must be 6 parts separated by ":"
    if ( !$mac_addr ) {
        return 0;
    }
    my @mac_numbers = split( /:/, $mac_addr );
    if ( @mac_numbers != 6 ) {
        return 0;
    }

    # each part of the MAC must be 2 chars long
    foreach my $mac_part (@mac_numbers) {
        if ( length($mac_part) != 2 ) {
            return 0;
        }
    }

    # check for correct hex characters
    if ( !chk_is_hex( join( '', @mac_numbers ) ) ) {
        return 0;
    }

    # all checks passed
    return 1;
}

=head2 Funktion chk_is_hex
 
Prfen eines Strings auf korrekte Hexadezimalzahl, d.h. darf nur aus
den Ziffern 0-9 und den Buchstaben a-f (auch Gro�uchstaben) bestehen.
 
=over 4
 
=item Parameter

 - String ohne Leerzeichen
 
=item Rckgabewert

True (1), wenn korrekter Hexadezimalstring, sonst False (0)
 
=back
 
=cut

sub chk_is_hex
{
    my ($hex) = @_;
    my $result = 0;

    if ( !defined $hex ) {
        return $result;
    }
    if (
        $hex =~ /^(\d|[a-f]|[A-F])        # begins with digit or hex-letter
                  (\d|[a-f]|[A-F])*$/x    # any number of digits or hex-letters
      )
    {
        $result = 1;
    }
    return $result;
}

=head2 Funktion chk_is_lcase
 
Prfen, ob ein String keine Gro�uchstaben enth�t.
 
=over 4
 
=item Parameter

 - String
 
=item Rckgabewert

True (1), wenn der String keine Gro�uchstaben enth�t, sonst False (0)
 
=back
 
=cut

sub chk_is_lcase
{
    my ($check_string) = @_;
    my $result = 0;
    if ( !defined $check_string ) {
        return 0;
    }
    if ( $check_string eq lc($check_string) ) {
        $result = 1;
    }
    return $result;
}

=head2 Funktion chk_split_dhcp_range
 
Liefert aus einem String, der zwei Kommaseparierte IP-Adressen enth�t,
ein Array der beiden IP-Adressen
 
=over 4
 
=item Parameter

 String, der eine IP-Range enth�t, wie sie z.B. in dbkDhcpRange (Objekt-
 klasse dbkLocation) verwendet wird, z.B. 10.201.5.1, 10.201.5.254
 
=item Rckgabewert

 Ein Array bestehend aus den beiden IP-Adressen
 
=back
 
=cut

sub chk_split_dhcp_range
{
    my ($ip_range) = @_;

    my $logger = Log::Log4perl->get_logger();

    if ( defined $ip_range ) {

        # remove all whitespace
        $ip_range =~ s/\s//g;
        my @ip_range = split( /,/, $ip_range );

        #TODO: should check if there are only two ip addresses (and if they are ip addresses at all)
        return @ip_range;
    }
    $logger->warn("split_dhcp_range() called with undef");
    return;
}

sub checkIp
{
    ## check a given IP Address
    my $ip_addr = shift;
    if ( !defined $ip_addr ) {
        return;
    }
    my @ip_numbers = split( /\./, $ip_addr );
    if ( @ip_numbers != 4 ) {
        return;
    }
    foreach my $number (@ip_numbers) {
        if ( $number =~ /^[^\d]+$/ ) {

            #not a number
            return;
        }
        if ( $number < 0 || $number > 255 ) {

            # not in the right range
            return;
        }
    }
    return $ip_addr;
}

# legacy wrapper
sub chk_ip
{
    return checkIp(@_);
}

1;
