#!/usr/bin/perl -w

#
# $Id: Constant.pm 7444 2011-10-06 14:09:38Z FSchumacher $
#

#
# package name
#
package SmartClient::Constant;
use strict;
use warnings;

#
# version  of the package
#
our $VERSION = 0.1;

=encoding utf8

=head1 NAME

C<SmartClient::Constant>



=head1 SYNOPSIS

package declares constants used by SmartClient programs


=head1 DESCRIPTION

=head2 Purpose

=head2 Realisation

=head2 Error handling

=head2 Context and Dependencies

=over 4

=item Context

=item Dependencies


=back



=head2 Interfaces

=begin comment

    Command line parameter are already descripied 
    at SYNOPSIS and usage

=end comment

=over 4

=item Programm

=item User

=back



=head2 Status

=begin comment

    descripes the TODOs/Errors for this programm

=end comment


=head2 Installation and other Notes



=head1 AUTHOR

    smartclient@dass-it.de




=head1 SEE ALSO



=cut

#
# module definition
#

use base 'Exporter';

=head1 symbols exported by default

- variable1
- function1
- function2

=cut

our @EXPORT = qw(
  SHELL_OK SHELL_FAILED
  PERM_LEVEL_UNDEF PERM_LEVEL_AUTO PERM_LEVEL_ADMIN PERM_LEVEL_USER
  PERM_UNDEF PERM_DENIED PERM_GRANTED
  ACTION_UNDEF ACTION_NOTHING ACTION_ADD ACTION_MOVE ACTION_MANUAL
  STATUS_UNDEF STATUS_ACTIV STATUS_INACTIV
  XMLRPC_SHARED_FUNCTION_TIMEOUT XMLRPC_OK XMLRPC_NO_SUCH_ID XMLRPC_NO_PERMISSION XMLRPC_ERROR_MANUAL XMLRPC_UNKNOWN_ACTION XMLRPC_LDAP_FAILURE XMLRPC_TIMEOUT XMLRPC_INCOMPLETE XMLRPC_INVALID_RESPONSE XMLRPC_FAULT
  SHARED_FUNCTION_DOES_NOT_EXIST SHARED_FUNCTION_STORE_RESULT_ERROR
  WORKSTATION_STATUS_FAILED_UPDATE WORKSTATION_STATUS_FAILED_CIS_UPDATE WORKSTATION_STATUS_FAILED_CIS_RESTART WORKSTATION_STATUS_FAILED_WOL WORKSTATION_STATUS_FAILED_SHUTDOWN
  WORKSTATION_STATUS_OK
  WORKSTATION_STATUS_START WORKSTATION_STATUS_SET_LOCALBOOT WORKSTATION_STATUS_WOL WORKSTATION_STATUS_CIS WORKSTATION_STATUS_CIS_NETBOOT WORKSTATION_STATUS_CIS_LOCALBOOT WORKSTATION_STATUS_REBOOTING
  LDAPCACHE_CACHE_OBJECTCLASS_NEW LDAPCACHE_CACHE_OBJECTCLASS_OK RPCXML_TIMEOUT
);

# =head1 symbols exported on request
#
# =cut
#
# @EXPORT_OK = qw( );

#
# used modules
#
#use strict;

#
# declarations
#

=head1 Constants

=cut

=head2 Shell return codes

return codes when simulating shell commands
(opposite to Perl true and false !)

=cut

use constant SHELL_OK     => 0;
use constant SHELL_FAILED => 1;

=head2 Permission Levels

permission levels for dislocated devices

    AUTO:   action on dislocated device can be performed without further user intervention
    ADMIN:  administrator must grant action
    USER:   user must grant action (currently not implemented)

=cut

use constant PERM_LEVEL_UNDEF => 0;
use constant PERM_LEVEL_AUTO  => 1;
use constant PERM_LEVEL_ADMIN => 2;
use constant PERM_LEVEL_USER  => 4;

=head2 Permissions

=cut

use constant PERM_UNDEF   => -1;
use constant PERM_DENIED  => 0;
use constant PERM_GRANTED => 1;

=head2 Actions

actions that can be performed on dislocated devices

=cut

use constant ACTION_UNDEF   => 0;
use constant ACTION_NOTHING => 1;
use constant ACTION_ADD     => 2;
use constant ACTION_MOVE    => 3;
use constant ACTION_MANUAL  => 4;

=head2 Status

status for dislocated devices

=cut

use constant STATUS_UNDEF   => 0;
use constant STATUS_ACTIV   => 1;
use constant STATUS_INACTIV => 2;

=head2 XML-RPC return codes

actions that can be performed on dislocated devices

=cut

use constant XMLRPC_SHARED_FUNCTION_TIMEOUT => -1;
use constant XMLRPC_OK                      => 0;
use constant XMLRPC_NO_SUCH_ID              => 1;
## perform actions on dislocated devices error codes
## (Leases2ldap)
use constant XMLRPC_NO_PERMISSION  => 2;
use constant XMLRPC_ERROR_MANUAL   => 4;
use constant XMLRPC_UNKNOWN_ACTION => 8;
## (Leases2ldap)
use constant XMLRPC_LDAP_FAILURE => 16;
## the timeout has expired on the server side
## (controld: waitForDeviceStatus)
use constant XMLRPC_TIMEOUT => 32;
## if response data is available,
## but code is not set
## (RpcXmlUtil: convertResult)
use constant XMLRPC_INCOMPLETE => 64;
## if response is not an XML-RPC object
## (RpcXmlUtil: convertResult)
use constant XMLRPC_INVALID_RESPONSE => 128;
## if response is fault, eg. HTTP timeout, connection error, ...
## (RpcXmlUtil: convertResult)
use constant XMLRPC_FAULT => 256;

use constant SHARED_FUNCTION_DOES_NOT_EXIST     => 1024;
use constant SHARED_FUNCTION_STORE_RESULT_ERROR => 2048;

=head2 Schedule Update status

Status values for schedule update

=cut

use constant WORKSTATION_STATUS_FAILED_UPDATE      => -5;
use constant WORKSTATION_STATUS_FAILED_CIS_UPDATE  => -4;
use constant WORKSTATION_STATUS_FAILED_CIS_RESTART => -3;
use constant WORKSTATION_STATUS_FAILED_WOL         => -2;
use constant WORKSTATION_STATUS_FAILED_SHUTDOWN    => -1;
use constant WORKSTATION_STATUS_OK                 => 0;
use constant WORKSTATION_STATUS_START              => 1;
use constant WORKSTATION_STATUS_SET_LOCALBOOT      => 2;
use constant WORKSTATION_STATUS_WOL                => 3;
use constant WORKSTATION_STATUS_CIS                => 4;
use constant WORKSTATION_STATUS_CIS_NETBOOT        => 5;
use constant WORKSTATION_STATUS_CIS_LOCALBOOT      => 6;
use constant WORKSTATION_STATUS_REBOOTING          => 7;

=head2 LdapCache

status values for the hierarchical cache in respect to objectclasses

=cut

use constant LDAPCACHE_CACHE_OBJECTCLASS_NEW => 1;
use constant LDAPCACHE_CACHE_OBJECTCLASS_OK  => 2;

=head2 Useless Constants

xmlrpc timeout is usually configured per config.pm, but is
troublesome to read from rpcxmlutil.pm. And http timeout is a constant anyway. 

=cut

use constant RPCXML_TIMEOUT => 120;
1;
