#!/usr/bin/perl -w
package SmartClient::FakeLdapResult;
use strict;

=head1 Name

SmartClient::FakeLdapResult

=head2 SVN Version

	$Id: FakeLdapResult.pm 7433 2011-10-06 12:26:47Z FSchumacher $

=head1 Synopsis

	use SmartClient::FakeLdapResult;
	use Net::LDAP;

	#read ldif into result
	my $result = SmartClient::FakeLdapResult->new('ldif' => 'test.ldif');

	#iterate over entries in $result
	foreach my $entry ( $result->entries ) {
		# do something with the entry
		print $entry->dn;
	}

=cut

use Net::LDAP;
use Net::LDAP::LDIF;

use base 'Net::LDAP::Message';

=head2 new

constructor for SmartClient::FakeLdapResult

=over 4

=item Parameter

	- ldif filename

=item Return

	- class-thingy

=back

=cut

sub new
{
    my $package = shift;
    my $data    = {
        'ldif' => undef,
        @_
    };

    if ( init($data) ) {
        return bless( $data, $package );
    } else {
        return;
    }
}

=head2 init

reads ldif file into object-thingy as entries

=over 4

=item Parameter

	- object-thingy

=item Return

	- object-thingy

=back

=cut

sub init
{
    my $data = shift;

    my @entries;
    $data->{'_count'} = 0;
    if ( $data->{'ldif'} ) {
        my $ldif = Net::LDAP::LDIF->new( $data->{'ldif'} );
        while ( not $ldif->eof() ) {
            my $entry = $ldif->read_entry();
            push @entries, $entry;
            ++$data->{'_count'};
        }
        $ldif->done();
    }
    $data->{'_entries'} = \@entries;

    return $data;
}

=head2 entries

simulates entries method from Net::LDAP::Entries

=over 4

=item Parameter

	- None

=item Return

	- array with entries stored in object

=back

=cut

sub entries
{
    my $self = shift;
    return @{ $self->{'_entries'} };
}

=head2 entry

simulates entry method of Net::LDAP::Result

=over 4

=item Parameter

integer index of entries in result

=item Return

C<Net::LDAP::Entry>

=back

=cut

sub entry
{
    my $self  = shift;
    my $index = shift;
    return ${ $self->{'_entries'} }[$index];

}

=head2 all_entries

=over 4

=item Parameter

=item Return

=back

=cut

sub all_entries
{
    my $self = shift;
    return @{ $self->{'_entries'} };
}

=head2 code

=over 4

=item Parameter

=item Return

=back

=cut

sub code
{
    my $self = shift;
    if ( $self->{'_entries'} ) {
        return 0;
    }
    return 1;
}

=head2 count

=over 4

=item Parameter

=item Return

=back

=cut

sub count
{
    my $self = shift;
    return $self->{'_count'};
}

1;
