#!/usr/bin/perl
package SmartClient::FakeNetLDAP;

use strict;
use warnings;

use base qw(Net::LDAP);
use Net::LDAP::LDIF;
use Log::Log4perl;
use SmartClient::FakeLdapResult;

our $AUTOLOAD;

=head2 new

=over 4

=item Parameter

=item Return

=back
=cut

sub new
{
    my $package     = shift;
    my $ldap_server = shift;

    my $self = {};
    $self->{'ldap_server'}     = $ldap_server;
    $self->{'ldap_connection'} = Net::LDAP->new($ldap_server);
    $self->{'ldap_searches'}   = {};
    $self->{'moddn_list'}      = [];
    $self->{'modify_list'}     = [];
    bless $self, $package;
}

=head2 search

=over 4

=item Parameter

=item Return

=back
=cut

sub search
{
    my $self   = shift;
    my %params = (
        base   => undef,
        scope  => 'sub',
        filter => '(objectClass=*)',
        attrs  => '*',
        @_
    );
    my $logger = Log::Log4perl->get_logger();
    my $ldif_file =
        $params{'base'} . '_'
      . $params{'scope'} . '_'
      . $params{'filter'} . '_'
      . $params{'attrs'};
    $ldif_file =~ s/[,\(\)*=]+/_/g;
    $ldif_file = './FakeSearches/' . $ldif_file;
    if ( -e $ldif_file ) {
        return SmartClient::FakeLdapResult->new( 'ldif' => $ldif_file );
    } else {
        my $result = $self->{'ldap_connection'}->search(%params);
        my $ldif_writer = Net::LDAP::LDIF->new( $ldif_file, 'w' );
        foreach my $entry ( $result->entries ) {
            $ldif_writer->write_entry($entry);
        }
        $ldif_writer->done();
        return $result;
    }
}

=head2 add

=over 4

=item Parameter

=item Return

=back
=cut

sub add
{
    my ($self, $dn, %params) = @_;
    my $logger = Log::Log4perl->get_logger();
    $logger->info('ldap_add');
    return SmartClient::FakeLdapResult->new();
}

=head2 moddn

=over 4

=item Parameter

=item Return

=back

=cut

sub moddn
{
    my $self   = shift;
    my $dn     = shift;
    my %params = (@_);
    my $logger = Log::Log4perl->get_logger();
    $logger->info("dn: $dn");
    my @moddn_list = @{ $self->{'moddn_list'} };
    push @moddn_list, [ $dn, \%params ];
    return SmartClient::FakeLdapResult->new();
}

=head2 modify

=over 4

=item Parameter

=item Return

=back

=cut

sub modify
{
    my $self   = shift;
    my $dn     = shift;
    my %params = (@_);
    my $logger = Log::Log4perl->get_logger();
    $logger->info("dn: $dn");
    my @modify_list = @{ $self->{'modify_list'} };
    push @modify_list, [ $dn, \%params ];
    return SmartClient::FakeLdapResult->new();
}

=head2 AUTOLOAD

=over 4

=item Parameter

=item Return

=back
=cut

sub AUTOLOAD
{
    my $self   = shift;
    my $type   = ref($self);
    my $name   = $::AUTOLOAD;
    my %params = (@_);
    print "$name";
    &{ $self->{'ldap_connection'}->$AUTOLOAD }(@_);
}

1;
