#!/usr/bin/perl -w
package SmartClient::MAC;
use strict;

=head1 NAME

=begin comment

Name des Moduls,Grobkonzept

=end comment

B<SmartClient::MAC>

=head1 SYNOPSIS

=begin comment

  kurze Beschreibung, was das Programm macht, Feinkonzept

=end comment

Objekt zur Ueberprfung von MAC-Adressen.

 SmartClient::MAC->new("MAC-Adresse");

=head1 DESCRIPTION

=begin comment

  muss fuer das Grobkonzept ausgefuellt werden

=end comment

=head2 Allg. Beschreibung und Aufgaben

Modul berprft eine MAC-Adresse.

=head2 Funktionsweise / Design / Architektur / Sprache

=over 4

=item Funktionsweise

B<SmartClient::MAC> bildet eine Instanz einer MAC-Adresse,
wenn diese gltig ist. Es wird dabei davon ausgegangen, da
eine MAC-Adresse folgenden Aufbau hat:
 xx:xxx:xx:xx:xx:xx
d.h. eine Abfolge von sechs hexadezimalen Zahlen durch Punkte getrennt. 
Die Zahlen knnen zwischen 0 und 0xFF liegen.




=item Architektur

Linux i386


=item Sprache

  Perl

=back

=head2 Sicherheitskonzept

keine Angabe

=head2 Einordnung / Abhngigkeiten / Voraussetzungen

=begin comment

Abhaenigkeiten von anderen Perlmodulen werden durch
require Statements sichergestellt.

=end comment

keine

=head2 Interfaces(Programm/Benutzer)

keine

=cut

our $VERSION = 0.1;

=head2 CVS Version

  $Id: MAC.pm 7442 2011-10-06 13:35:14Z FSchumacher $

=cut

=head2 Installation und sonstige Hinweise

=begin comment

Feinkonzept

=end comment

Modul mu im @INC -Pfad stehen.

=head1 AUTHOR

=begin comment

name and e-mail

=end comment

Frank Balzer mailto:frank.balzer@suse.de

SuSE Linux Solutions AG mailto: rheinruhr@suse.de

=head1 SEE ALSO


=cut

=head1 exportierte Module, Methoden, Variablen

=over 4

=item Variable

S<  >B<MAC>


=back

=cut

=head1 Modul

=begin comment

Beschreibung des Moduls

=end comment

S<  >B<SmartClient::MAC> { I<IP> =E<gt> I<MAC-Adresse>};

 # Beispiel

 $ip = SmartClient::MAC->new("10:ab:e1:b1:11:22");

=cut

# constructor
sub new
{
    my ( $package, $mac ) = @_;
    $mac = parse($mac);
    if ( !$mac ) {
        return $mac;
    }
    return bless { 'MAC' => $mac, }, $package;
}

=head1 Methoden

=head2 parse()

S<  >B<parse()>

Methode zum berprfen der MAC-Adresse.

Wird bei der Bildung einer Instanz bentigt, um zu entscheiden ob eine
gltige Instanz gebildet wird oder nicht.

=cut

sub parse
{
    my $mac_addr = shift;
    my @mac_numbers = split( /:/, $mac_addr );
    if ( @mac_numbers != 6 ) {
        $mac_addr = undef;
        return $mac_addr;
    }
    foreach my $number (@mac_numbers) {
        if ( hex($number) < 0 || hex($number) > 0xff ) {
            $mac_addr = undef;
            return $mac_addr;
        }
    }
    return $mac_addr;
}

=head2 returnAddress()

S<  >B<returnAddress()>

gibt die IP-Adresse einer gltigen Instanz zurck.

=cut

sub returnAddress
{
    my ($this) = @_;
    return $this->{'MAC'};
}

1;
