#!/usr/bin/perl -w

#
# Template zur Dokumentation von Perlmodulen
#

#enter package name z.B. DSAP::Modulname,
package SmartClient::Rsync;

=head1 NAME

=begin comment

Grobkonzept

=end comment

=head1 SYNOPSIS

=begin comment

  kurze Beschreibung, was das Programm macht, Feinkonzept

=end comment

Das Modul SmartClient::Rsync ist eine Klasse zum Aufruf von rsync mit Wiederholungs-
versuchen, mehreren Servern etc. 

=head1 DESCRIPTION

=begin comment

  muss fuer das Grobkonzept ausgefuellt werden

=end comment

=head2 Allg. Beschreibung und Aufgaben

 - Aufrufen von rsync
 - definierbare Anzahl von Wiederholungsversuchen
 - definierbare Wartezeit zwischen den Wiederholungsversuchen
 - timeout

=head2 Funktionsweise / Design / Architektur / Sprache

=over 4

=item Funktionsweise

=item Architektur

Linux, i386

=begin comment

# Allgemein

=end comment

=item Sprache

  Perl

=back



=head2 Sicherheitskonzept

=head2 Einordnung / Abhaengigkeiten / Voraussetzungen

=begin comment

Abhaenigkeiten von anderen Perlmodulen werden durch
use oder require Statements sichergestellt.

=end comment

=cut

use strict;
use Data::Dumper;
use Log::Log4perl;
use RPC::XML;
use RPC::XML::Client;
use SmartClient::RpcXmlUtil;

# version of the package

my $VERSION = 0.1;

=head2 CVS Version

  $Id: Rsync.pm 7574 2011-11-07 13:17:39Z FSchumacher $


=head2 Installation und sonstige Hinweise

=begin comment

Feinkonzept

=end comment

=head1 AUTHOR

Stephan Duehr

=head1 SEE ALSO

=cut

sub new
{
    my $package = shift;    # packagename

    # Definition of defaults
    my $data = {

        # reference to array of rsync server names
        'SERVERS' => undef,

        # XML-RPC connection object
        # for getting/releasing access permissions for rsync servers
        'XML_RPC_CONN' => undef,

        # Timeout for function getRsyncAccessPermission
        'XML_RPC_WAIT_TIMEOUT' => 60,

        # METASRC is used for special HDCACHE handling.
        # In all other cases,
        # the rsync protocol (rsync::) is expected
        'METASRC' => undef,

        # source path
        'SOURCE' => undef,

        # target path
        'TARGET' => undef,

        # reference to array of rsync options
        'OPTIONS' => undef,

        # number of retries for each server
        'RETRIES' => 3,

        # seconds to wait between retries
        'DELAY' => 10,
        ## definies special server from SERVERS,
        ## where other options can apply
        'FALLBACK_SERVER'  => "rsync",
        'FALLBACK_OPTIONS' => undef,
        @_
    };

    # TODO: check parameters

    # return the object-hash
    return bless($data, $package);
}

sub run
{
    my $logger = Log::Log4perl->get_logger();

    my $self     = shift;
    my $cli      = $self->{XML_RPC_CONN};
    my $time_out = $self->{XML_RPC_WAIT_TIMEOUT};

    my $rsync_ret;
    my $break = 0;
    my $wait  = 1;
    while ( $wait > 0 ) {
        $wait = 0;

        foreach my $rsync_server ( @{ $self->{SERVERS} } ) {

            $logger->debug("RSYNC server: $rsync_server");

            my $rsync_source;
            if ( $self->{METASRC} eq "HDCACHE" ) {
                $rsync_source = "/" . $self->{SOURCE};
            } else {
                $rsync_source = $rsync_server . "::" . $self->{SOURCE};
            }

            my @options = @{ $self->{OPTIONS} };
            if (    ( lc($rsync_server) eq $self->{'FALLBACK_SERVER'} )
                and ( $self->{FALLBACK_OPTIONS} ) )
            {
                @options = @{ $self->{FALLBACK_OPTIONS} };
            }

            # checks, if rsync source is available
            if ( checkRsyncSource($rsync_source) ) {

                my $retries = $self->{RETRIES};
                my $rsync_perm;
                $rsync_ret = 255;

                my @rsync_command =
                  ( "rsync", @options, $rsync_source, $self->{TARGET} );

                if ($cli) {

                    # XML-RPC connection exists,
                    # check RSync access permission

                    # TODO: error checking
                    $rsync_perm = $cli->send_request(
                        "status.getRsyncAccessPermission",
                        RPC::XML::string->new("$rsync_server")
                    )->value;

                    if ( $rsync_perm eq "WAIT" ) {
                        $wait++;
                    } elsif ( $rsync_perm eq "BREAK" ) {
                        $break++;
                    }

                    $logger->debug(
                        "getRsyncAccessPermission($rsync_server) = $rsync_perm"
                    );

                    # else should be OK
                } else {
                    $logger->debug(
                        "no connection for getRsyncAccessPermission($rsync_server), assuming OK"
                    );
                    $rsync_perm = "OK";
                }

                # if RSync access permission "OK" -> update
                # Check RSync access permission end
                if ( $rsync_perm eq "OK" ) {

                    while ( ( $retries > 0 ) && ( $rsync_ret != 0 ) ) {
                        $rsync_ret = system(@rsync_command) / 256;
                        if ( $rsync_ret == 0 ) {
                            setKnutStatus(
                                'rpc_xml_daemon' => $cli,
                                'message' =>
                                  "RsyncAccessPermission($rsync_server): OK"
                            );
                        }
                        $retries--;
                        sleep( $self->{DELAY} );
                    }

                    if ($cli) {

                        # Now release connection to RSync host with XML-RPC service
                        my $resp = $cli->send_request(
                            "status.releaseRsyncAccessPermission",
                            RPC::XML::string->new("$rsync_server")
                        )->value;
                        if ( !$resp ) {
                            $logger->error(
                                "failed to release Rsync access permission via XML-RPC for $rsync_server"
                            );
                        }
                    }

                    if ( $rsync_ret == 0 ) {
                        return $rsync_ret;
                    } else {
                        $logger->info(
                            "failed to run " . join( " ", @rsync_command ) );
                    }
                }
            }
        }    # end of foreach loop over SERVERS
        if ( $wait > 0 ) {
            setKnutStatus(
                'rpc_xml_daemon' => $cli,
                'message' =>
                  "waiting ($time_out) for rsync access permission to "
                  . join( ", ", @{ $self->{SERVERS} } )
            );
            sleep $time_out;
        }
    }    # end of while ($wait > 0)

    if ($break) {
        setKnutStatus(
            'rpc_xml_daemon' => $cli,
            'message' => "failed to get rsync access permission. Rsync break"
        );
    }

    # we only get here if all servers and all retries failed
    return $rsync_ret;
}

=head2 checkRsyncSource( $ )

checks if a rsync source is available

=over 4

=item Parameter

 - rsync source (eg. rsync::images/refImage102)

=item Return

  boolean: 
    - true, if rsync source is available
    - false, otherwise

=back

=cut

sub checkRsyncSource( $ )
{
    my $rsync_source = shift;

    # Bugfix um eine Kompatibilitaet mit der Rsync-Server-Version 2.6.2 herzustellen
    # Die Client Version 3.x kann NICHT korrekt den Status der Rsync-Freigaben ermitteln
    $rsync_source =~ s/\:\:.*/\:\:/g;
    my $timeout       = 3;
    my @rsync_command = ( "rsync", "--timeout=$timeout", $rsync_source );
    my $rsync_ret     = system(@rsync_command) / 256;
    return ( $rsync_ret == 0 );
}

1;
