#!/usr/bin/perl -w

#
# $Id: ScheduleUpdate.pm 8722 2013-04-12 13:56:07Z p0422584 $
#

#
# package name
#
package SmartClient::ScheduleUpdate;
use strict;
use warnings;

#
# version  of the package
#
our $VERSION = 0.1;

=encoding utf8

=head1 NAME

C<ScheduleUpdate>



=head1 SYNOPSIS

function used for schedule update


=head1 DESCRIPTION

=head2 Purpose

=head2 Realisation

=head2 Error handling

=head2 Context and Dependencies

=over 4

=item Context

=item Dependencies


=back



=head2 Interfaces

=begin comment

    Command line parameter are already descripied 
    at SYNOPSIS and usage

=end comment

=over 4

=item Programm

=item User

=back



=head2 Status

=begin comment

    descripes the TODOs/Errors for this programm

=end comment


=head2 Installation and other Notes



=head1 AUTHOR

    smartclient@dass-it.de




=head1 SEE ALSO



=cut

#
# used modules
#
use Data::Dumper;
use Log::Log4perl;
use Log::Log4perl::Level;
use Net::Ping;
use SmartClient::Constant;
use SmartClient::Util;
use SmartClient::LdapUtil;
use SmartClient::RpcXmlUtil;

#
# module definition
#

use base qw(Exporter);

=head1 symbols exported by default

- variable1
- function1
- function2

=cut

our @EXPORT =
  qw( set_location_localboot update_clients wait_for_update finish_update setStatus checkCis send_wake_on_lan ping_workstation check_rsync sendSshCommand reboot_ws halt_ws );

=head1 symbols exported on request

=cut

our @EXPORT_OK = qw( );

#
# declarations
#

=head1 Constants

=head1 Global Variables




=head1 Functions



=head2 set_location_localboot($)

set boot option for location to localboot

=over 4

=item Parameter

    - location_dn

=item Return

    integer: 0: success

=back

=cut

sub set_location_localboot
{
    my $logger = Log::Log4perl->get_logger();

    my %params = (
        'ldap'           => undef,
        'location_dn'    => undef,
        'device_type_dn' => undef,
        @_
    );

    my $ldap           = $params{'ldap'};
    my $location_dn    = $params{'location_dn'};
    my $device_type_dn = $params{'device_type_dn'};

    my $location_standards_dn = ldap_construct_standards_name(
        'LDAP'         => $ldap,
        'DN'           => $location_dn,
        'DEVICETYPEDN' => $device_type_dn,
    );

    if ( !$location_standards_dn ) {
        $logger->error(
            "failed to get standards DN for location $location_dn and device type $device_type_dn"
        );
        return -1;
    }

    my $code = $ldap->set_values(
        DN     => $location_standards_dn,
        VALUES => {
            objectClass   => 'dbkUpdate',
            dbkDoRsync    => 'LAN',
            dbkDoTftpBoot => 'LOCALBOOT'
        }
    );

    if ($code) {
        $logger->error(
            "failed to set $location_dn ($location_standards_dn) to localboot");
    }

    # note: $code is 0 on success.
    return $code;
}

=head2 setStatus( % )

=cut

sub setStatus( % )
{
    my $logger = Log::Log4perl->get_logger();
    my %params = (
        'ldap'           => undef,
        'dn'             => undef,
        'status_value'   => undef,
        'status_message' => undef,
        'log'            => $DEBUG,
        @_
    );

    my $ldap       = $params{'ldap'};
    my $dn         = $params{'dn'};
    my $error_code = $params{'status_value'};
    my $error_msg  = $params{'status_message'};
    my $log        = $params{'log'};

    my $msg;
    my @timestring = localtime( time() );
    if ( $error_code && $error_msg ) {
        $msg = sprintf(
            "%s, %02d.%02d.%04d %02d:%02d:%02d, %s",
            $error_code, $timestring[3],
            $timestring[4] + 1,
            $timestring[5] + 1900,
            $timestring[2], $timestring[1], $timestring[0], $error_msg
        );

    } elsif ($error_msg) {
        $msg = sprintf(
            "0, %02d.%02d.%04d %02d:%02d:%02d, %s",
            $timestring[3],
            $timestring[4] + 1,
            $timestring[5] + 1900,
            $timestring[2], $timestring[1], $timestring[0], $error_msg
        );
    } else {
        $msg = "0";
    }

    $logger->log( $log, "$dn: $error_msg ($error_code)" );

    my $result = $ldap->modify( $dn, replace => { dbkStatus => $msg } );
    if ( $result->code ) {
        $logger->error( $result->error );
        return -2;
    }

    return 1;
}

=head2 checkCis( % )


=cut

sub checkCis( % )
{

    #my $logger      = Log::Log4perl->get_logger();

    my %params = (
        'dn'          => undef,
        'test_daemon' => undef,
        'timeout'     => 60,
        @_
    );

    my $cis_dn      = $params{'dn'};
    my $test_daemon = $params{'test_daemon'};
    my $timeout     = $params{'timeout'};

    return retry(
        'function'  => \&check_rsync,
        'aref_args' => [ $cis_dn, $test_daemon ],
        'timeout'   => $timeout
    );
}

=head2 send_wake_on_lan( % )

       sendet ein WOL magic Packet.
       benutzt wol.pl
            Parameter dn einer Workstation,macAddress

       WakeOnLan-Programm und WOL_TIMEOUT werden aus Config-Datei bestimmt
       gibt Zeit des Sendens zurück

=cut

sub send_wake_on_lan( % )
{
    my $logger = Log::Log4perl->get_logger();

    my %params = (
        'config'           => undef,
        'dn'               => undef,
        'entry'            => undef,
        'test_daemon'      => undef,
        'location_netmask' => undef,
        @_
    );

    my $config           = $params{'config'};
    my $dn               = $params{'dn'};
    my $entry            = $params{'entry'};
    my $test_daemon      = $params{'test_daemon'};
    my $location_netmask = $params{'location_netmask'};

    my $macAddress   = $entry->{'macAddress'};
    my $ipHostNumber = $entry->{'ipHostNumber'};

    my $result = 0;

    if ($test_daemon) {

        $logger->info("calling send_wake_on_lan for $dn");

        my $response =
          $test_daemon->send_request( 'sendWol', RPC::XML::string->new($dn) );

        if ( ref($response) ) {
            $result = $response->value();
        } else {

            # this has happend several times
            # when writing/logging to STDOUT/STDERR !
            $logger->error("no valid response for $dn");
        }

    } else {
        $logger->info(
            "Trying WOL: $dn ($ipHostNumber/$location_netmask) MAC $macAddress"
        );
        wakeOnLan(
            'ip_address'  => $ipHostNumber,
            'netmask'     => $location_netmask,
            'mac_address' => $macAddress,
            'count'       => 10
        );
    }
    return $result;
}

=head2 ping_workstation()

    Ping eine Workstation an
        parameter Dn einer Workstation

    gibt 1 bei Erfolg, 0 bei Misserfolg zurück

=cut

sub ping_workstation( $;$ )
{
    my $logger      = Log::Log4perl->get_logger();
    my $dn          = shift;
    my $test_daemon = shift;

    if ( !$dn ) {
        $logger->error("parameter missing");
        return;
    }
    my $dns_name = ldap_dn2dnsname($dn);
    my $result;

    if ($test_daemon) {

        #$logger->info("calling sendPing with $dn");
        my $response =
          $test_daemon->send_request( 'sendPing', RPC::XML::string->new($dn) );

        if ( ref($response) ) {
            $result = $response->value();
        } else {
            $logger->error( "no response for $dn (" . Dumper($response) . ")" );
            return;
        }
    } else {
        my $ping = Net::Ping->new( "icmp", 10, 16 );
        $result = $ping->ping($dns_name);
    }

    $logger->debug("ping $dns_name ($dn) returns [$result]");

    return $result;
}

=head2 check_rsync($;$)

    checks if a connection to an rsync server is possible

    =over 4
    
    =item Parameter
    
        - dn of rsync server
        - [schedule_update_testdaemon]
    
    =item Return
    
        boolean
    
    =back

=cut

sub check_rsync( $;$ )
{
    my $logger      = Log::Log4perl->get_logger();
    my $dn          = shift;
    my $test_daemon = shift;

    if ( !$dn ) {
        $logger->error("parameter missing");
        return;
    }
    my $dns_name = ldap_dn2dnsname($dn);
    my $result;

    if ($test_daemon) {

        #         #$logger->info("calling sendPing with $dn");
        #         my $response =
        #           $test_daemon->send_request( 'checkRsync',
        #             RPC::XML::string->new($dn) );
        #
        #         if ( ref($response) ) {
        #             $result = $response->value();
        #         } else {
        #             $logger->error( "no response for $dn (" . Dumper($response) . ")" );
        #             return;
        #         }

        # TODO: implement on test_daemon. So long, it always returns true
        $result = 1;

    } else {

        # connects to an rsync server
        if ( ping_workstation( $dn, $test_daemon ) ) {
            my ( $return_code, $output, $stderr ) =
              run_command_timeout("rsync ${dns_name}::");

            # convert shell return code (0 on success)
            # to boolean code (0 for failure)
            $result = !$return_code;
        } else {

            # workstation was not alive, so no rsync check
            $result = 0;
        }
    }

    $logger->debug("rsync $dns_name ($dn) returns [$result]");

    return $result;
}

=head2 sendSshCommand( $$;$ )

executes a command on a workstation using ssh

=over 4

=item Parameter

    $dn
    $remote_command
    [ $test_daemon ]

=item Return

    ssh return code (or undef)

=back

=cut

sub sendSshCommand( $$;$ )
{
    my $logger = Log::Log4perl->get_logger();

    my $dn             = shift;
    my $remote_command = shift;
    my $test_daemon    = shift;

    my $return_code;
    my $stdout;
    my $stderr;

    my $dns_name = ldap_dn2dnsname($dn);

    if ($test_daemon) {

        my $response = $test_daemon->send_request(
            'sendSshCommand',
            RPC::XML::string->new($dn),
            RPC::XML::string->new($remote_command)
        );

        if ( ref($response) ) {
            $return_code = $response->value();
        } else {
            $logger->error("no response for $dn: $remote_command");
            return;
        }

    } else {
        my $command =
          'ssh -o \'UserKnownHostsFile /dev/null\' -o \'StrictHostkeyChecking=no\' -q root@';
        $command .= $dns_name . ' "' . $remote_command . '"';
        ( $return_code, $stdout, $stderr ) = run_command_timeout($command);
    }

    #$logger->debug(
    #    "SSH command $remote_command on $dn ($dns_name) returns: ".join("\n",@output)."[$return_code]");

    return $return_code;
}

=head2 reboot_ws()

    startet eine workstation neu,
         Parameter dn der Workstation

=cut

sub reboot_ws( $;$ )
{
    my $dn          = shift;
    my $test_daemon = shift;
    return sendSshCommand( $dn, "reboot", $test_daemon );
}

=head2 halt_ws()

        startet eine workstation neu,
                 Parameter dn der Workstation

=cut

sub halt_ws( $;$ )
{
    my $dn          = shift;
    my $test_daemon = shift;
    return sendSshCommand( $dn, "halt -p", $test_daemon );
}

=head2 update_clients()

   Initiiert das Update von Clients
   Parameter: Referenz auf Hash von Workstations
              CIS

=cut

sub update_clients
{
    my $logger = Log::Log4perl->get_logger();

    my %params = (
        'ldap'              => undef,
        'workstations'      => undef,
        'href_workstations' => undef,
        'cis_dn'            => undef,
        'refimage_version'  => undef,
        'config'            => undef,
        'location_netmask'  => "255.255.255.0",
        'test_daemon'       => undef,
        'status_daemon'     => undef,
        'activation_time'   => 30,
        'boot_sleep_time'   => 2,
        @_
    );

    my $ldap                   = $params{'ldap'};
    my @workstations_to_update = @{ $params{'workstations'} };
    my $ldap_workstations      = $params{'href_workstations'};
    my $cis_dn                 = $params{'cis_dn'};
    my $refimage_version       = $params{'refimage_version'};
    my $config                 = $params{'config'};
    my $location_netmask       = $params{'location_netmask'};
    my $test_daemon            = $params{'test_daemon'};
    my $status_daemon          = $params{'status_daemon'};
    my $activation_time        = $params{'activation_time'};
    my $boot_sleep_time        = $params{'boot_sleep_time'};

    my @started_workstations = ();
    foreach my $ws_dn (@workstations_to_update) {

        # only update if workstation is not the CIS
        if ( $ws_dn ne $cis_dn ) {

            # $ldap_workstations->{$ws_dn} exists only
            # if $ws_dn is in the correct location,
            # uses correct refImage and  devicetype
            my $ws_entry = $ldap_workstations->{$ws_dn};
            if ( !$ws_entry ) {
                $logger->warn(
                    "no information available for workstation $ws_dn. Skipped");
            } else {

                # setUpdate has already been called
                # to set the workstations to netboot.
                # but setUpdate only changes
                # the LDAP entry,
                # so we wait,
                # until the services (DHCP, DNS) are updated.
                # We also check,
                # if workstation has been updated
                # in between (manually?)
                my $resp = waitForDeviceStatus(
                    'rpc_xml_daemon'  => $status_daemon,
                    'device_dn'       => $ws_dn,
                    'netboot'         => 1,
                    'dbkImageVersion' => $refimage_version,
                    'timeout'         => $activation_time,
                );
                if ( ( !$resp ) or ( $resp->{'code'} != XMLRPC_OK ) ) {
                    $logger->error(
                        "workstation $ws_dn is expected to be set to netboot. This is not the case. Skipped"
                    );
                } elsif ( $resp->{dbkImageVersion} == $refimage_version ) {
                    $logger->info(
                        "workstation $ws_dn is already up-to-date. Skipped");
                    next;
                } else {
                    push @started_workstations, $ws_dn;
                    if ( ping_workstation( $ws_dn, $test_daemon ) ) {

                        # ws is powered on
                        setStatus(
                            'ldap'           => $ldap,
                            'dn'             => $ws_dn,
                            'status_value'   => WORKSTATION_STATUS_REBOOTING,
                            'status_message' => "reboot",
                            'log'            => $INFO
                        );
                        reboot_ws( $ws_dn, $test_daemon );

                    } else {

                        # sending wol

                        # DEBUG
                        #$logger->debug( "workstations: " . Dumper( $ldap_workstations ) );
                        #$logger->debug( "DN: $ws_dn" );
                        #$logger->debug( "entry: " . Dumper( $ws_entry ) );

                        my $mac_address = $ws_entry->{'macAddress'};
                        send_wake_on_lan(
                            'config'           => $config,
                            'dn'               => $ws_dn,
                            'entry'            => $ws_entry,
                            'test_daemon'      => $test_daemon,
                            'location_netmask' => $location_netmask
                        );
                    }

                    # sleep a short while, to prevent,
                    # that all workstation boot at the same time
                    sleep($boot_sleep_time);
                }
            }
        }

    }
    return @started_workstations;
}

=head2 wait_for_update(%)

    wait for workstations to complete update 
    (current dbkImageVersion and in productive mode (rsync is on)),
    but don't wait longer than 'max_update_time' seconds.

=over 4

=item Parameter


=item Return

    array:
        - aref of succesfully updated workstation
        - aref where update has failed

=back


=cut

sub wait_for_update(%)
{
    my $logger = Log::Log4perl->get_logger();

    my %params = (
        'ldap'             => undef,
        'location_dn'      => undef,
        'refimage_dn'      => undef,
        'refimage_version' => undef,
        'devicetype_dn'    => undef,
        'cis_dn'           => undef,
        'workstations'     => undef,
        'status_daemon'    => undef,
        'test_daemon'      => undef,
        ## 3600s = 1h
        'max_update_time' => 3600,
        'check_time'      => 60,
        @_
    );

    my $ldap                   = $params{'ldap'};
    my $location_dn            = $params{'location_dn'};
    my $refimage_dn            = $params{'refimage_dn'};
    my $refimage_version       = $params{'refimage_version'};
    my $devicetype_dn          = $params{'devicetype_dn'};
    my $cis_dn                 = $params{'cis_dn'};
    my %workstations_to_update = map { $_ => 1 } @{ $params{'workstations'} };
    my $status_daemon          = $params{'status_daemon'};
    my $test_daemon            = $params{'test_daemon'};
    my $max_update_time        = $params{'max_update_time'};
    my $check_time             = $params{'check_time'};

    # remove CIS from workstation hash (it is already up-to-date)
    delete( $workstations_to_update{$cis_dn} );

    my $start_time = time();

    my %workstations_updated;
    my @workstations_ready;

    # loop until all workstions are updated or timeout
    while ( %workstations_to_update
        and ( time() < ( $start_time + $max_update_time ) ) )
    {

        $logger->debug("sleeping ${check_time}s");
        sleep($check_time);

        foreach my $ws_dn ( keys %workstations_to_update ) {
            if ( !exists $workstations_updated{$ws_dn} ) {
                my $device_status = getDeviceStatus(
                    'rpc_xml_daemon' => $status_daemon,
                    'device_dn'      => $ws_dn,
                );
                if ($device_status) {
                    my $ws_image_ver = $device_status->{'dbkImageVersion'};
                    if ( $ws_image_ver eq $refimage_version ) {
                        $workstations_updated{$ws_dn} = time();
                        $logger->info(
                            "$ws_dn has now the current image version $refimage_version"
                        );
                    }
                }
            }
        }

        foreach my $ws_dn ( keys %workstations_updated ) {
            if ( exists $workstations_to_update{$ws_dn} ) {
                if ( check_rsync( $ws_dn, $test_daemon ) ) {
                    setStatus(
                        'ldap'         => $ldap,
                        'dn'           => $ws_dn,
                        'status_value' => WORKSTATION_STATUS_OK,
                        'status_message' =>
                          "successfully updated to $refimage_version",
                        'log' => $INFO
                    );
                    push( @workstations_ready, $ws_dn );
                    delete( $workstations_to_update{$ws_dn} );
                } elsif ( $workstations_updated{$ws_dn} + 300 < time() ) {

                    # TODO make 300 to Config->get(REBOOT_TIME) maybe
                    delete( $workstations_to_update{$ws_dn} );
                    $logger->warn(
                        "$ws_dn could not be checked for rsync after successful update"
                    );
                }
            }

        }
    }

    my @workstations_not_updated = keys %workstations_to_update;
    return ( \@workstations_ready, \@workstations_not_updated );
}

=head2 finish_update( % )

=cut

sub finish_update
{
    my $logger = Log::Log4perl->get_logger();

    my %params = (
        'ldap'              => undef,
        'ldap_workstations' => undef,
        'cis_dn'            => undef,
        'not_updated'       => undef,
        'cis_leave_on'      => undef,
        'test_daemon'       => undef,
        @_
    );

    my $ldap              = $params{'ldap'};
    my $ldap_workstations = $params{'ldap_workstations'};
    my $cis_dn            = $params{'cis_dn'};
    my $aref_not_updated  = $params{'not_updated'};
    my $cis_leave_on      = $params{'cis_leave_on'};
    my $test_daemon       = $params{'test_daemon'};

    foreach my $ws_dn ( keys %{$ldap_workstations} ) {

        # ignore cis
        next if ( $ws_dn eq $cis_dn );

        # ignore not yet updated workstations
        next if ( grep( /$ws_dn/, @{$aref_not_updated} ) );

        if ( ping_workstation( $ws_dn, $test_daemon ) ) {
            halt_ws( $ws_dn, $test_daemon );
        }
    }

    # shutdown CIS only if no problems
    # and no not yet updated workstation remain
    if ( !( $cis_leave_on or @{$aref_not_updated} ) ) {
        halt_ws( $cis_dn, $test_daemon );
    } else {
        $logger->warn( "failed to update workstations "
              . join( "; ", @{$aref_not_updated} ) )
          if ( @{$aref_not_updated} );
        $logger->info("CIS $cis_dn will not be shut down");
    }

    return $aref_not_updated;
}

1;
